@@ -1,5 +1,5 @@
 /*
- * Copyright 2013  Rinat Ibragimov
+ * Copyright 2013-2014  Rinat Ibragimov
  *
  * This file is part of libvdpau-va-gl
  *
@@ -179,6 +179,7 @@ vdpDeviceCreateX11(Display *display_orig, int screen, VdpDevice *device,
     if (NULL == data)
         return VDP_STATUS_RESOURCES;
 
+    glx_ctx_lock(); // use glx lock to serialize X calls
     data->type = HANDLETYPE_DEVICE;
     data->display = display;
     data->display_orig = display_orig;   // save supplied pointer too
@@ -195,6 +196,7 @@ vdpDeviceCreateX11(Display *display_orig, int screen, VdpDevice *device,
         (PFNGLXBINDTEXIMAGEEXTPROC)glXGetProcAddress((GLubyte *)"glXBindTexImageEXT");
     data->fn.glXReleaseTexImageEXT =
         (PFNGLXRELEASETEXIMAGEEXTPROC)glXGetProcAddress((GLubyte *)"glXReleaseTexImageEXT");
+    glx_ctx_unlock();
 
     if (!data->fn.glXBindTexImageEXT || !data->fn.glXReleaseTexImageEXT) {
         traceError("error (%s): can't get glXBindTexImageEXT address\n");
@@ -247,17 +249,8 @@ vdpDeviceCreateX11(Display *display_orig, int screen, VdpDevice *device,
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-    char *tex_data = malloc(watermark_width * watermark_height * 4);
-    if (tex_data) {
-        const char *s = watermark_data;
-        char *d = tex_data;
-        for (int k = 0; k < watermark_width * watermark_height; k ++) {
-            *d++ = 255; *d++ = 255; *d++ = 255; *d++ = *s++;
-        }
-        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, watermark_width, watermark_height, 0, GL_BGRA,
-                     GL_UNSIGNED_BYTE, tex_data);
-        free(tex_data);
-    }
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, watermark_width, watermark_height, 0, GL_BGRA,
+                 GL_UNSIGNED_BYTE, watermark_data);
     glFinish();
 
     *device = handle_insert(data);
