@@ -1,5 +1,5 @@
 /*
- * Copyright 2013  Rinat Ibragimov
+ * Copyright 2013-2014  Rinat Ibragimov
  *
  * This file is part of libvdpau-va-gl
  *
@@ -23,6 +23,20 @@
 #include "trace.h"
 #include "watermark.h"
 
+
+struct task_s {
+    struct timespec         when;
+    uint32_t                clip_width;
+    uint32_t                clip_height;
+    VdpOutputSurface        surface;
+    unsigned int            wipe_tasks;
+    VdpPresentationQueue    queue_id;
+};
+
+static GAsyncQueue *async_q = NULL;
+static pthread_t    presentation_thread_id;
+
+
 static
 VdpTime
 timespec2vdptime(struct timespec t)
@@ -145,31 +159,26 @@ recreate_pixmaps_if_geometry_changed(VdpPresentationQueueTargetData *pqTargetDat
 
 static
 void
-do_presentation_queue_display(VdpPresentationQueueData *pqData)
+do_presentation_queue_display(struct task_s *task)
 {
-    assert(pqData->queue.used > 0);
-
-    const int entry = pqData->queue.head;
+    VdpPresentationQueueData *pqData =
+        handle_acquire(task->queue_id, HANDLETYPE_PRESENTATION_QUEUE);
+    if (!pqData)
+        return;
     VdpDeviceData *deviceData = pqData->deviceData;
-    VdpOutputSurface surface = pqData->queue.item[entry].surface;
-    const uint32_t clip_width = pqData->queue.item[entry].clip_width;
-    const uint32_t clip_height = pqData->queue.item[entry].clip_height;
-
-    // remove first entry from queue
-    pqData->queue.used --;
-    pqData->queue.freelist[pqData->queue.head] = pqData->queue.firstfree;
-    pqData->queue.firstfree = pqData->queue.head;
-    pqData->queue.head = pqData->queue.item[pqData->queue.head].next;
+    const VdpOutputSurface surface = task->surface;
+    const uint32_t clip_width = task->clip_width;
+    const uint32_t clip_height = task->clip_height;
 
     VdpOutputSurfaceData *surfData = handle_acquire(surface, HANDLETYPE_OUTPUT_SURFACE);
-    if (surfData == NULL)
+    if (surfData == NULL) {
+        handle_release(task->queue_id);
         return;
+    }
 
     glx_ctx_lock();
     recreate_pixmaps_if_geometry_changed(pqData->targetData);
-    glx_ctx_unlock();
-    glx_ctx_push_global(deviceData->display, pqData->targetData->glx_pixmap,
-                        pqData->targetData->glc);
+    glXMakeCurrent(deviceData->display, pqData->targetData->glx_pixmap, pqData->targetData->glc);
 
     const uint32_t target_width  = (clip_width > 0)  ? clip_width  : surfData->width;
     const uint32_t target_height = (clip_height > 0) ? clip_height : surfData->height;
@@ -206,7 +215,7 @@ do_presentation_queue_display(VdpPresentationQueueData *pqData)
         glMatrixMode(GL_TEXTURE);
         glLoadIdentity();
 
-        glColor3f(0.8, 0.08, 0.35);
+        glColor4f(1.0, 1.0, 1.0, 0.2);
         glBegin(GL_QUADS);
             glTexCoord2i(0, 0);
             glVertex2i(target_width - watermark_width, target_height - watermark_height);
@@ -224,21 +233,18 @@ do_presentation_queue_display(VdpPresentationQueueData *pqData)
 
     glFinish();
     GLenum gl_error = glGetError();
-    glx_ctx_pop();
 
     x11_push_eh();
-    glx_ctx_lock();
-    XSync(deviceData->display, False);
     XCopyArea(deviceData->display, pqData->targetData->pixmap, pqData->targetData->drawable,
               pqData->targetData->plain_copy_gc, 0, 0, target_width, target_height, 0, 0);
     XSync(deviceData->display, False);
     int x11_err = x11_pop_eh();
-    glx_ctx_unlock();
     if (x11_err != Success) {
         char buf[200] = { 0 };
         XGetErrorText(deviceData->display, x11_err, buf, sizeof(buf));
         traceError("warning (%s): caught X11 error %s\n", __func__, buf);
     }
+    glx_ctx_unlock();
 
     struct timespec now;
     clock_gettime(CLOCK_REALTIME, &now);
@@ -253,64 +259,83 @@ do_presentation_queue_display(VdpPresentationQueueData *pqData)
     }
 
     handle_release(surface);
+    handle_release(task->queue_id);
 
     if (GL_NO_ERROR != gl_error) {
         traceError("error (%s): gl error %d\n", __func__, gl_error);
     }
 }
 
+gint
+compare_func(gconstpointer a, gconstpointer b, gpointer user_data)
+{
+    const struct task_s *task_a = a;
+    const struct task_s *task_b = b;
+
+    if (task_a->when.tv_sec < task_b->when.tv_sec)
+        return -1;
+    else if (task_a->when.tv_sec > task_b->when.tv_sec)
+        return 1;
+    else if (task_a->when.tv_nsec < task_b->when.tv_nsec)
+        return -1;
+    else if (task_a->when.tv_nsec > task_b->when.tv_nsec)
+        return 1;
+    else
+        return 0;
+}
+
 static
 void *
 presentation_thread(void *param)
 {
-    VdpPresentationQueueData *pqData = (VdpPresentationQueueData *)param;
+    GQueue *int_q = g_queue_new();          // internal queue of task, always sorted
 
-    pthread_mutex_lock(&pqData->queue_mutex);
-    pthread_barrier_wait(&pqData->thread_start_barrier);
     while (1) {
-        struct timespec now;
-        clock_gettime(CLOCK_REALTIME, &now);
-        struct timespec target_time = now;
-
-        while (1) {
-            int ret;
-            ret = pthread_cond_timedwait(&pqData->new_work_available,
-                                         &pqData->queue_mutex, &target_time);
-            if (ret != 0 && ret != ETIMEDOUT) {
-                traceError("error (%s): pthread_cond_timedwait failed with code %d\n", __func__,
-                           ret);
-                pthread_mutex_unlock(&pqData->queue_mutex);
-                return NULL;
-            }
+        gint64 timeout;
+        struct task_s *task = g_queue_peek_head(int_q);
 
+        if (task) {
+            // internal queue have a task
             struct timespec now;
             clock_gettime(CLOCK_REALTIME, &now);
-            if (pqData->thread_state == 1) {
-                pqData->thread_state = 2;
-                pthread_mutex_unlock(&pqData->queue_mutex);
-                return NULL;
-            }
-            if (pqData->queue.head != -1) {
-                struct timespec ht = vdptime2timespec(pqData->queue.item[pqData->queue.head].t);
-                if (now.tv_sec > ht.tv_sec ||
-                    (now.tv_sec == ht.tv_sec && now.tv_nsec > ht.tv_nsec))
-                {
-                    // break loop and process event
-                    break;
-                } else {
-                    // sleep until next event
-                    target_time = ht;
-                }
-            } else {
-                // queue empty, no work to do. Wait for next event
-                target_time = now;
-                target_time.tv_sec += 1;
+            timeout = (task->when.tv_sec - now.tv_sec) * 1000 * 1000 +
+                      (task->when.tv_nsec - now.tv_nsec) / 1000;
+            if (timeout <= 0) {
+                // task is ready to go
+                g_queue_pop_head(int_q); // remove it from queue
+
+                // run the task
+                do_presentation_queue_display(task);
+                g_slice_free(struct task_s, task);
+                continue;
             }
+        } else {
+            // no tasks in queue, sleep for a while
+            timeout = 1000 * 1000; // one second
         }
 
-        // do event processing
-        do_presentation_queue_display(pqData);
+        task = g_async_queue_timeout_pop(async_q, timeout);
+        if (task) {
+            if (task->wipe_tasks) {
+                // create new internal queue by filtering old
+                GQueue *new_q = g_queue_new();
+                while (!g_queue_is_empty(int_q)) {
+                    struct task_s *t = g_queue_pop_head(int_q);
+                    if (t->queue_id != task->queue_id)
+                        g_queue_push_tail(new_q, t);
+                }
+                g_queue_free(int_q);
+                int_q = new_q;
+
+                g_slice_free(struct task_s, task);
+                continue;
+            }
+            g_queue_insert_sorted(int_q, task, compare_func, NULL);
+        }
     }
+
+    g_queue_free(int_q);
+    return NULL;
 }
 
 VdpStatus
@@ -346,44 +371,20 @@ vdpPresentationQueueCreate(VdpDevice device, VdpPresentationQueueTarget presenta
     data->bg_color.green = 0.0;
     data->bg_color.blue = 0.0;
     data->bg_color.alpha = 0.0;
-    data->thread_state = 0;
 
     ref_device(deviceData);
     ref_pq_target(targetData);
     *presentation_queue = handle_insert(data);
 
-    // initialize queue
-    data->queue.head = -1;
-    data->queue.used = 0;
-    for (unsigned int k = 0; k < PRESENTATION_QUEUE_LENGTH; k ++) {
-        data->queue.item[k].next = -1;
-        // other fields are zero due to calloc
+    // initialize queue and launch worker thread
+    if (!async_q) {
+        async_q = g_async_queue_new();
+        pthread_create(&presentation_thread_id, NULL, presentation_thread, data);
     }
-    for (unsigned int k = 0; k < PRESENTATION_QUEUE_LENGTH - 1; k ++)
-        data->queue.freelist[k] = k + 1;
-    data->queue.freelist[PRESENTATION_QUEUE_LENGTH - 1] = -1;
-    data->queue.firstfree = 0;
 
-    pthread_cond_init(&data->new_work_available, NULL);
-    pthread_mutex_init(&data->queue_mutex, NULL);
-    pthread_barrier_init(&data->thread_start_barrier, NULL, 2);
-
-    // launch worker thread
-    pthread_mutex_lock(&data->queue_mutex);
-    pthread_create(&data->worker_thread, NULL, presentation_thread, data);
     handle_release(device);
     handle_release(presentation_queue_target);
 
-    pthread_mutex_unlock(&data->queue_mutex);
-    pthread_yield();
-
-    // wait till worker thread passes first lock
-    pthread_barrier_wait(&data->thread_start_barrier);
-    // wait till worker thread unlocks queue_mutex. That means, it started listening to
-    // conditional variable
-    pthread_mutex_lock(&data->queue_mutex);
-    pthread_mutex_unlock(&data->queue_mutex);
-
     return VDP_STATUS_OK;
 }
 
@@ -395,15 +396,12 @@ vdpPresentationQueueDestroy(VdpPresentationQueue presentation_queue)
     if (NULL == pqData)
         return VDP_STATUS_INVALID_HANDLE;
 
-    pthread_mutex_lock(&pqData->queue_mutex);
-    pqData->thread_state = 1;   // send termination request
-    pthread_cond_broadcast(&pqData->new_work_available);
-    pthread_mutex_unlock(&pqData->queue_mutex);
+    struct task_s *task = g_slice_new0(struct task_s);
+    task->when = vdptime2timespec(0);   // as early as possible
+    task->queue_id = presentation_queue;
+    task->wipe_tasks = 1;
+    g_async_queue_push(async_q, task);
 
-    pthread_join(pqData->worker_thread, NULL);
-
-    pthread_cond_destroy(&pqData->new_work_available);
-    pthread_barrier_destroy(&pqData->thread_start_barrier);
     handle_expunge(presentation_queue);
     unref_device(pqData->deviceData);
     unref_pq_target(pqData->targetData);
@@ -473,18 +471,6 @@ vdpPresentationQueueDisplay(VdpPresentationQueue presentation_queue, VdpOutputSu
     if (NULL == pqData)
         return VDP_STATUS_INVALID_HANDLE;
 
-    // push work to queue
-    while (pqData && pqData->queue.used >= PRESENTATION_QUEUE_LENGTH) {
-        // wait while queue is full
-        // TODO: check for deadlock here
-        handle_release(presentation_queue);
-        usleep(10*1000);
-        pqData = handle_acquire(presentation_queue, HANDLETYPE_PRESENTATION_QUEUE);
-    }
-
-    if (NULL == pqData)
-        return VDP_STATUS_INVALID_HANDLE;
-
     VdpOutputSurfaceData *surfData = handle_acquire(surface, HANDLETYPE_OUTPUT_SURFACE);
     if (NULL == surfData) {
         handle_release(presentation_queue);
@@ -496,50 +482,27 @@ vdpPresentationQueueDisplay(VdpPresentationQueue presentation_queue, VdpOutputSu
         return VDP_STATUS_HANDLE_DEVICE_MISMATCH;
     }
 
-    pthread_mutex_lock(&pqData->queue_mutex);
-    pqData->queue.used ++;
-    int new_item = pqData->queue.firstfree;
-    assert(new_item != -1);
-    pqData->queue.firstfree = pqData->queue.freelist[new_item];
+    struct task_s *task = g_slice_new0(struct task_s);
+
+    task->when = vdptime2timespec(earliest_presentation_time);
+    task->clip_width = clip_width;
+    task->clip_height = clip_height;
+    task->surface = surface;
+    task->queue_id = presentation_queue;
 
-    pqData->queue.item[new_item].t = earliest_presentation_time;
-    pqData->queue.item[new_item].clip_width = clip_width;
-    pqData->queue.item[new_item].clip_height = clip_height;
-    pqData->queue.item[new_item].surface = surface;
     surfData->first_presentation_time = 0;
     surfData->status = VDP_PRESENTATION_QUEUE_STATUS_QUEUED;
 
-    // keep queue sorted
-    if (pqData->queue.head == -1 ||
-        earliest_presentation_time < pqData->queue.item[pqData->queue.head].t)
-    {
-        pqData->queue.item[new_item].next = pqData->queue.head;
-        pqData->queue.head = new_item;
-    } else {
-        int ptr = pqData->queue.head;
-        int prev = ptr;
-        while (ptr != -1 && pqData->queue.item[ptr].t <= earliest_presentation_time) {
-            prev = ptr;
-            ptr = pqData->queue.item[ptr].next;
-        }
-        pqData->queue.item[new_item].next = ptr;
-        pqData->queue.item[prev].next = new_item;
-    }
-    pthread_mutex_unlock(&pqData->queue_mutex);
-
     if (global.quirks.log_pq_delay) {
         struct timespec now;
         clock_gettime(CLOCK_REALTIME, &now);
         surfData->queued_at = timespec2vdptime(now);
     }
 
-    pthread_mutex_lock(&pqData->queue_mutex);
-    pthread_cond_broadcast(&pqData->new_work_available);
-    pthread_mutex_unlock(&pqData->queue_mutex);
+    g_async_queue_push(async_q, task);
 
     handle_release(presentation_queue);
     handle_release(surface);
-
     return VDP_STATUS_OK;
 }
 
