@@ -1,5 +1,5 @@
 /*
- * Copyright 2013  Rinat Ibragimov
+ * Copyright 2013-2014  Rinat Ibragimov
  *
  * This file is part of libvdpau-va-gl
  *
@@ -11,7 +11,7 @@
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <libswscale/swscale.h>
-#include "pixfmt_compat.h"
+#include "compat.h"
 #include "shaders.h"
 #include <stdlib.h>
 #include <string.h>
@@ -134,12 +134,11 @@ vdpVideoSurfaceDestroy(VdpVideoSurface surface)
 
     glx_ctx_push_thread_local(deviceData);
     glDeleteTextures(1, &videoSurfData->tex_id);
-
     GLenum gl_error = glGetError();
+    glx_ctx_pop();
 
     if (GL_NO_ERROR != gl_error) {
         traceError("error (%s): gl error %d\n", __func__, gl_error);
-        glx_ctx_pop();
         handle_release(surface);
         return VDP_STATUS_ERROR;
     }
@@ -162,7 +161,6 @@ vdpVideoSurfaceDestroy(VdpVideoSurface surface)
         free(videoSurfData->u_plane);
     // do not free videoSurfData->v_plane, it's just pointer into the middle of u_plane
 
-    glx_ctx_pop();
     unref_device(deviceData);
     handle_expunge(surface);
     free(videoSurfData);
@@ -520,6 +518,8 @@ vdpVideoSurfacePutBitsYCbCr_glsl(VdpVideoSurface surface, VdpYCbCrFormat source_
     glMatrixMode(GL_TEXTURE);
     glLoadIdentity();
 
+    glDisable(GL_BLEND);
+
     switch (source_ycbcr_format) {
     case VDP_YCBCR_FORMAT_NV12:
         glUseProgram(deviceData->shaders[glsl_NV12_RGBA].program);
