@@ -3,9 +3,9 @@
 // creating and destroying couple of VdpDevice from different threads
 // caused deadlocks and crashes
 
-#include "vdpau-init.h"
 #include <stdio.h>
 #include <pthread.h>
+#include "tests-common.h"
 
 VdpDevice device1;
 VdpDevice device2;
@@ -14,20 +14,22 @@ void *
 thread_1(void *param)
 {
     (void)param;
-    ASSERT_OK(vdp_device_destroy(device1));
+    ASSERT_OK(vdpDeviceDestroy(device1));
     return NULL;
 }
 
 int main(void)
 {
-    ASSERT_OK(vdpau_init_functions(&device1, NULL, 0));
-    ASSERT_OK(vdpau_init_functions(&device2, NULL, 0));
+    Display *dpy = get_dpy();
+
+    ASSERT_OK(vdpDeviceCreateX11(dpy, 0, &device1, NULL));
+    ASSERT_OK(vdpDeviceCreateX11(dpy, 0, &device2, NULL));
 
     pthread_t thread_id_1;
     pthread_create(&thread_id_1, NULL, thread_1, NULL);
     pthread_join(thread_id_1, NULL);
 
-    ASSERT_OK(vdp_device_destroy(device2));
+    ASSERT_OK(vdpDeviceDestroy(device2));
     printf("pass\n");
     return 0;
 }
